--[[ RaidBuffs ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
}
local misc = {
  ["desc"] = "RaidBuffs!";
  ["alpha"] = true;
  ["GUI"] = {
  };
}
local function getOptionIndexByName(name)
  if misc and misc["GUI"] then
    for i, v in ipairs(misc["GUI"]) do
      if v["opt"] == name then
        return i
      end
    end
  end
  return 0
end

local M = FixR:NewModule("RaidBuffs", DefaultO, misc)

local frame = nil
local frameEvents = {};
local setOptionBoolean
local moving = false

local _G, NUM_LE_RAID_BUFF_TYPES, GetRaidBuffTrayAuraInfo = _G, NUM_LE_RAID_BUFF_TYPES, GetRaidBuffTrayAuraInfo

local function initFrame()
  frame = CreateFrame("Frame", "FixR_RaidBuffs", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("LOW")
  frame:SetSize(200, 40)
  frame:SetAlpha(0.5)
  frame:Show()
  
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  
  frame.executeCounter = 0
  frame.inCombat = false
  
  frame.buffTexts = {};
  lastText = nil
  --for i = 1, (NUM_LE_RAID_BUFF_TYPES + 1) do
  for i = 1, NUM_LE_RAID_BUFF_TYPES do
    --frame.buffTexts[i] = frame:CreateFontString(nil, "BACKGROUND")
    --frame.buffTexts[i]:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE")
    --frame.buffTexts[i] = frame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    frame.buffTexts[i] = frame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    --frame.buffTexts[i]:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -11 * (i-1))
    if lastText then
      frame.buffTexts[i]:SetPoint("TOPLEFT", lastText, "TOPRIGHT", 0, 0)
    else
      frame.buffTexts[i]:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
    end
    if i <= NUM_LE_RAID_BUFF_TYPES then
      frame.buffTexts[i]:SetText(_G["RAID_BUFF_"..i])
    --else
    --  frame.buffTexts[i]:SetText("Last second: 0")
    end
    frame.buffTexts[i]:SetTextColor(1, 1, 1, 1)
    lastText = frame.buffTexts[i]
  end
  
  frame.calculate = function(self)
    if self.executeCounter == 0 then
      self.executeCounter = 1
      self.timeSinceLastUpdate = 0
      self:calculateExecute()
    else
      self.executeCounter = self.executeCounter + 1
    end
  end
  frame.calculateExecute = function(self)
    for i=1,NUM_LE_RAID_BUFF_TYPES do
      local name, rank, texture, duration, expiration, spellId, slot = GetRaidBuffTrayAuraInfo(i)
      --local buff = ConsolidatedBuffsTooltip["Buff"..i]
      if name then
        --[[
        buff.index = slot;
        buff.name = name;
        buff.icon:SetTexture(texture);
        buff.label:SetFontObject(GameFontNormalSmall);
        numBufs = numBufs + 1
        --]]
        self.buffTexts[i]:SetTextColor(0, 1, 0, 1)
      else
        --[[
        buff.index = nil;
        buff.name = nil;
        buff.icon:SetTexture("Interface\\Common\\buff-bg");
        if (bit.band(buffmask, mask ) > 0) then
          buff.label:SetFontObject(GameFontHighlightSmall);			
        else
          buff.label:SetFontObject(GameFontDisableSmall);			
        end
        buff.label:SetText(buff.labelString);
        --]]
        self.buffTexts[i]:SetTextColor(1, 0, 0, 1)
      end
      --mask = bit.lshift(mask, 1);
    end
  end
  
  frame.timeSinceLastUpdate = 0
  frame.timerInterval = 3
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    if self.timeSinceLastUpdate >= self.timerInterval then
      --self.buffTexts[NUM_LE_RAID_BUFF_TYPES + 1]:SetText(format("Last %d seconds: %d", self.timerInterval, self.executeCounter))
      if self.executeCounter > 0 then
        self.executeCounter = 0
        self:calculateExecute()
      end
      
      self.timeSinceLastUpdate = 0
    end
  end
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
end

function frameEvents:UNIT_AURA(...)
  --print("UNIT_AURA", ...)
  self:calculate()
end
function frameEvents:GROUP_ROSTER_UPDATE(...)
  --print("GROUP_ROSTER_UPDATE", ...)
  self:calculate()
end
function frameEvents:PLAYER_SPECIALIZATION_CHANGED(...)
  --print("PLAYER_SPECIALIZATION_CHANGED", ...)
  self:calculate()
end
function frameEvents:PLAYER_REGEN_DISABLED(...)
  self.inCombat = true
  self:SetAlpha(0.5)
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self:SetAlpha(0.5)
end

function M:Load()
  if not frame then
    initFrame()
  end
  --[[
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k)
  end
  --]]
  frame:RegisterUnitEvent("UNIT_AURA", "player", "vehicle")
  frame:RegisterEvent("GROUP_ROSTER_UPDATE")
  frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
  frame:RegisterEvent("PLAYER_REGEN_DISABLED")
  frame:RegisterEvent("PLAYER_REGEN_ENABLED")
  frame:Show()
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k)
    end
    frame:Hide()
  end
end

setOptionBoolean = function(optName, arg2, forceUpdateGUI)
  local bln, updateGUI = nil, true
  if arg2 == 1 then
    bln = true
    updateGUI = false
  elseif arg2 == 0 then
    bln = false
    updateGUI = false
  else --toggle
    bln = not (M.O[optName] or false)
  end
  M.O[optName] = bln
  FixR:Echo("RaidBuffs: "..optName..": "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  if updateGUI or forceUpdateGUI then
    FixR:SetModuleGUIOption(M, getOptionIndexByName(optName), bln)
  end
  return bln, updateGUI --return <new_state>, <gui_needed_update>
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      frame:SetMovable(true)
      frame:EnableMouse(true)
      frame:RegisterForDrag("LeftButton")
    end
    FixR:Echo("RaidBuffs: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  end
end